VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cSteelCollection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private m_col As Collection

Public Enum enumSteelFields
    SteelName
End Enum

' Initialize a new instance of this class
Private Sub Class_Initialize()
    Set m_col = New Collection
End Sub

' Add a new steel of the collection
Public Sub Add(ByVal st As cSteel)
    Call m_col.Add(st)
End Sub

' Remove the specified steel form the collection
Public Sub Remove(ByVal strName As String)
Dim st As cSteel
Dim i As Integer

    For i = 1 To m_col.Count
        Set st = m_col.Item(i)
        If st.Name = strName Then
            Call m_col.Remove(i)
            Exit Sub
        End If
    Next
End Sub

' Clear the collection
Public Sub Clear()
    Set m_col = New Collection
End Sub

' Return the item with the specified index
Public Property Get Item(ByVal index As Long) As cSteel
    Set Item = m_col.Item(index)
End Property

' Replace the item with the specified index
Public Property Set Item(ByVal index As Long, ByVal st As cSteel)
    Call m_col.Remove(index)
    If index <= m_col.Count Then
        Call m_col.Add(st, , index)
    Else
        Call m_col.Add(st)
    End If
End Property

Public Function GetSteelByName(ByVal strName As String) As cSteel
Dim st As cSteel
    
    For Each st In m_col
        If st.Name = strName Then
            Set GetSteelByName = st
            Exit Function
        End If
    Next
End Function

' Load the collection from the specified path
Public Sub LoadFromPath(ByVal strPath As String)
Dim strFile As String
Dim st As cSteel

'    On Error Resume Next
    strFile = Dir$(strPath & "\*.dat", vbHidden + vbSystem + vbReadOnly)
    Do While Len(strFile)
        Set st = New cSteel
        Call st.LoadFromFile(strFile)
        Call m_col.Add(st)
        strFile = Dir$
    Loop

End Sub

' Return the number of steel in the collection
Public Property Get Count() As Integer
    Count = m_col.Count
End Property

' Sort the collection on the specified order and by the specified steel field
Public Function Sort(ByVal eOrder As enumSortOrders, ByVal eField As enumSteelFields) As Long
    Dim lIncrement      As Long
    Dim lMainLoop       As Long
    Dim lSubLoop        As Long
    Dim st              As cSteel
    Dim lComp           As Long
    
    ' Implementation of Shell Sort algorithm using
    ' + 1 * 3 increment.
    ' Get inital Sort increment
    If (m_col.Count > 2) Then
        lIncrement = GetSuitableSortInitialIncrement(m_col.Count)
        Do Until lIncrement < 1
            For lMainLoop = lIncrement + 1 To m_col.Count
                Set st = m_col(lMainLoop)
                ' Loop form MainLoop-Increment to 0
                For lSubLoop = (lMainLoop - lIncrement) To 1 Step -lIncrement
                    ' Retrouver l'index de la colonne  classer en cours
                    lComp = Compare(eField, m_col(lSubLoop), st)
                    Select Case True
                    Case (lComp = 0)
                        ' No Swap
                        Exit For
                    Case (eOrder = enumSortOrders.Ascending)
                        If lComp < 1 Then
                            Exit For
                        End If
                    Case Else
                        If lComp > -1 Then
                            Exit For
                        End If
                    End Select
                    
                    Set Me.Item(lSubLoop + lIncrement) = m_col(lSubLoop)
                Next lSubLoop
                Set Me.Item(lSubLoop + lIncrement) = st
            Next lMainLoop
            ' Get next shell sort increment value:
            lIncrement = lIncrement - 1
            lIncrement = lIncrement \ 3
        Loop

    ElseIf m_col.Count = 2 Then
        ' For only two items just check whether the second should
        ' be swapped with the first:
        Set st = m_col(1)
        ' Retrouver l'index de la colonne  classer en cours
        lComp = Compare(eField, m_col(2), st)
        Select Case True
        Case (lComp = 0)
            ' No Swap
        Case (eOrder = enumSortOrders.Ascending)
            If (lComp < 1) Then
                ' swap
                Call m_col.Remove(1)
                Call m_col.Add(st)
            End If
        Case Else
            If (lComp > -1) Then
                ' swap
                Call m_col.Remove(1)
                Call m_col.Add(st)
            End If
        End Select
    End If

    Sort = m_col.Count
    
End Function

' Part of the implementation of Shell Sort algorithm using
' + 1 * 3 increment strategy.  This function returns the
' largest increment based on +1*3 which is less than the
' sort size.
Private Function GetSuitableSortInitialIncrement(ByVal lSortSize As Long) As Long
    Dim lR      As Long
    Dim lRLast  As Long
    
    lRLast = 1
    lR = 1
    Do While lR < lSortSize
        lRLast = lR
        lR = lR * 3 + 1
    Loop
    GetSuitableSortInitialIncrement = lRLast
End Function

Private Function Compare(ByVal eField As enumSteelFields, ByVal steel1 As cSteel, ByVal steel2 As cSteel) As Long
    Select Case eField
        Case enumSteelFields.SteelName
            Compare = StrComp(steel1.Name, steel2.Name, vbTextCompare)
                        
        Case Else
            Debug.Assert False
            Compare = 0
    End Select
End Function

